@@ -1,12 +1,12 @@
 /* keylist.c
- * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003,
- *               2004 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006,
+ *               2008, 2012 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
  * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * GnuPG is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -61,11 +60,11 @@ public_key_list( STRLIST list )
 {
   if(opt.with_colons)
     {
-      byte trust_model,marginals,completes,cert_depth;
+      byte trust_model,marginals,completes,cert_depth,min_cert_level;
       ulong created,nextcheck;
 
       read_trust_options(&trust_model,&created,&nextcheck,
-			 &marginals,&completes,&cert_depth);
+			 &marginals,&completes,&cert_depth,&min_cert_level);
 
       printf("tru:");
 
@@ -81,6 +80,8 @@ public_key_list( STRLIST list )
 	    printf("c");
 	  if(cert_depth!=opt.max_cert_depth)
 	    printf("d");
+	  if(min_cert_level!=opt.min_cert_level)
+	    printf("l");
 	}
 
       printf(":%d:%lu:%lu",trust_model,created,nextcheck);
@@ -95,6 +96,13 @@ public_key_list( STRLIST list )
       printf("\n");
     }
 
+  /* We need to do the stale check right here because it might need to
+     update the keyring while we already have the keyring open.  This
+     is very bad for W32 because of a sharing violation. For real OSes
+     it might lead to false results if we are later listing a keyring
+     which is associated with the inode of a deleted file.  */
+  check_trustdb_stale ();
+
   if( !list )
     list_all(0);
   else
@@ -104,6 +112,8 @@ public_key_list( STRLIST list )
 void
 secret_key_list( STRLIST list )
 {
+    check_trustdb_stale ();
+
     if( !list )
 	list_all(1);
     else  /* List by user id */
@@ -124,7 +134,7 @@ print_seckey_info (PKT_secret_key *sk)
 	      pubkey_letter (sk->pubkey_algo),
 	      keystr(keyid), datestr_from_sk (sk), p);
     
-  m_free (p);
+  xfree (p);
 }
 
 /* Print information about the public key.  With FP passed as NULL,
@@ -155,9 +165,63 @@ print_pubkey_info (FILE *fp, PKT_public_key *pk)
                 nbits_from_pk (pk), pubkey_letter (pk->pubkey_algo),
                 keystr(keyid), datestr_from_pk (pk), p);
 
-  m_free (p);
+  xfree (p);
 }
 
+
+/* Print basic information of a secret key including the card serial
+   number information. */
+void
+print_card_key_info (FILE *fp, KBNODE keyblock)
+{
+  KBNODE node;
+  int i;
+
+  for (node = keyblock; node; node = node->next ) 
+    {
+      if (node->pkt->pkttype == PKT_SECRET_KEY
+          || (node->pkt->pkttype == PKT_SECRET_SUBKEY) )
+        {
+          PKT_secret_key *sk = node->pkt->pkt.secret_key;
+          
+          tty_fprintf (fp, "%s%c  %4u%c/%s  ",
+		       node->pkt->pkttype == PKT_SECRET_KEY? "sec":"ssb",
+                       (sk->protect.s2k.mode==1001)?'#':
+                       (sk->protect.s2k.mode==1002)?'>':' ',
+		       nbits_from_sk (sk),
+		       pubkey_letter (sk->pubkey_algo),
+		       keystr_from_sk(sk));
+          tty_fprintf (fp, _("created: %s"), datestr_from_sk (sk));
+          tty_fprintf (fp, "  ");
+          tty_fprintf (fp, _("expires: %s"), expirestr_from_sk (sk));
+          if (sk->is_protected && sk->protect.s2k.mode == 1002)
+            {
+              tty_fprintf (fp, "\n                      ");
+              tty_fprintf (fp, _("card-no: ")); 
+              if (sk->protect.ivlen == 16
+                  && !memcmp (sk->protect.iv, "\xD2\x76\x00\x01\x24\x01", 6))
+                { 
+                  /* This is an OpenPGP card. */
+                  for (i=8; i < 14; i++)
+                    {
+                      if (i == 10)
+                        tty_fprintf (fp, " ");
+                      tty_fprintf (fp, "%02X", sk->protect.iv[i]);
+                    }
+                }
+              else
+                { /* Something is wrong: Print all. */
+                  for (i=0; i < sk->protect.ivlen; i++)
+                    tty_fprintf (fp, "%02X", sk->protect.iv[i]);
+                }
+            }
+          tty_fprintf (fp, "\n");
+        }
+    }
+}
+
+
+
 /* Flags = 0x01 hashed 0x02 critical */
 static void
 status_one_subpacket(sigsubpkttype_t type,size_t len,int flags,const byte *buf)
@@ -268,71 +332,55 @@ show_keyserver_url(PKT_signature *sig,int indent,int mode)
 void
 show_notation(PKT_signature *sig,int indent,int mode,int which)
 {
-  const byte *p;
-  size_t len;
-  int seq=0,crit;
   FILE *fp=mode?log_stream():stdout;
+  struct notation *nd,*notations;
 
   if(which==0)
     which=3;
 
+  notations=sig_to_notation(sig);
+
   /* There may be multiple notations in the same sig. */
+  for(nd=notations;nd;nd=nd->next)
+    {
+      if(mode!=2)
+	{
+	  int has_at=!!strchr(nd->name,'@');
 
-  while((p=enum_sig_subpkt(sig->hashed,SIGSUBPKT_NOTATION,&len,&seq,&crit)))
-    if(len>=8)
-      {
-	int n1,n2;
+	  if((which&1 && !has_at) || (which&2 && has_at))
+	    {
+	      int i;
+	      const char *str;
 
-	n1=(p[4]<<8)|p[5];
-	n2=(p[6]<<8)|p[7];
+	      for(i=0;i<indent;i++)
+		putchar(' ');
 
-	if(8+n1+n2!=len)
-	  {
-	    log_info(_("WARNING: invalid notation data found\n"));
-	    continue;
-	  }
+	      if(nd->flags.critical)
+		str=_("Critical signature notation: ");
+	      else
+		str=_("Signature notation: ");
+	      if(mode)
+		log_info("%s",str);
+	      else
+		printf("%s",str);
+	      /* This is all UTF8 */
+	      print_utf8_string(fp,nd->name,strlen(nd->name));
+	      fprintf(fp,"=");
+	      print_utf8_string(fp,nd->value,strlen(nd->value));
+	      fprintf(fp,"\n");
+	    }
+	}
 
-	if(mode!=2)
-	  {
-	    int has_at=!!memchr(p+8,'@',n1);
+      if(mode)
+	{
+	  write_status_buffer(STATUS_NOTATION_NAME,
+			      nd->name,strlen(nd->name),0);
+	  write_status_buffer(STATUS_NOTATION_DATA,
+			      nd->value,strlen(nd->value),50);
+	}
+    }
 
-	    if((which&1 && !has_at) || (which&2 && has_at))
-	      {
-		int i;
-		const char *str;
-
-		for(i=0;i<indent;i++)
-		  putchar(' ');
-
-		/* This is UTF8 */
-		if(crit)
-		  str=_("Critical signature notation: ");
-		else
-		  str=_("Signature notation: ");
-		if(mode)
-		  log_info("%s",str);
-		else
-		  printf("%s",str);
-		print_utf8_string(fp,p+8,n1);
-		fprintf(fp,"=");
-
-		if(*p&0x80)
-		  print_utf8_string(fp,p+8+n1,n2);
-		else
-		  fprintf(fp,"[ %s ]",_("not human readable"));
-
-		fprintf(fp,"\n");
-	      }
-	  }
-
-	if(mode)
-	  {
-	    write_status_buffer ( STATUS_NOTATION_NAME, p+8   , n1, 0 );
-	    write_status_buffer ( STATUS_NOTATION_DATA, p+8+n1, n2, 50 );
-	  }
-      }
-  else
-    log_info(_("WARNING: invalid notation data found\n"));
+  free_notation(notations);
 }
 
 static void
@@ -501,7 +549,8 @@ print_capabilities (PKT_public_key *pk, PKT_secret_key *sk, KBNODE keyblock)
   if(pk || (sk && sk->protect.s2k.mode!=1001))
     {
       unsigned int use = pk? pk->pubkey_usage : sk->pubkey_usage;
-    
+          int c_printed = 0;
+
       if ( use & PUBKEY_USAGE_ENC )
         putchar ('e');
 
@@ -509,9 +558,19 @@ print_capabilities (PKT_public_key *pk, PKT_secret_key *sk, KBNODE keyblock)
 	{
 	  putchar ('s');
 	  if( pk? pk->is_primary : sk->is_primary )
-	    putchar ('c');
+            {
+              putchar ('c');
+              /* The PUBKEY_USAGE_CERT flag was introduced later and
+                 we used to always print 'c' for a primary key.  To
+                 avoid any regression here we better track whether we
+                 printed 'c' already.  */
+              c_printed = 1;
+            }
 	}
 
+      if ( (use & PUBKEY_USAGE_CERT) && !c_printed )
+        putchar ('c');
+
       if ( (use & PUBKEY_USAGE_AUTH) )
         putchar ('a');
     }
@@ -537,6 +596,8 @@ print_capabilities (PKT_public_key *pk, PKT_secret_key *sk, KBNODE keyblock)
 			if(pk->is_primary)
 			  cert = 1;
 		      }
+                    if ( pk->pubkey_usage & PUBKEY_USAGE_CERT )
+                      cert = 1;
                     if ( (pk->pubkey_usage & PUBKEY_USAGE_AUTH) )
                       auth = 1;
                 }
@@ -554,6 +615,8 @@ print_capabilities (PKT_public_key *pk, PKT_secret_key *sk, KBNODE keyblock)
 			if(sk->is_primary)
 			  cert = 1;
 		      }
+                    if ( (sk->pubkey_usage & PUBKEY_USAGE_CERT) )
+                        cert = 1;
                     if ( (sk->pubkey_usage & PUBKEY_USAGE_AUTH) )
                         auth = 1;
                 }
@@ -726,7 +789,7 @@ list_keyblock_print ( KBNODE keyblock, int secret, int fpr, void *opaque )
 	else if(pk->has_expired)
 	  {
 	    printf(" [");
-	    printf(_("expired: %s)"),expirestr_from_pk(pk));
+	    printf(_("expired: %s"),expirestr_from_pk(pk));
 	    printf("]");
 	  }
 	else if(pk->expiredate)
@@ -760,7 +823,7 @@ list_keyblock_print ( KBNODE keyblock, int secret, int fpr, void *opaque )
 	if( node->pkt->pkttype == PKT_USER_ID && !opt.fast_list_mode ) {
 	    PKT_user_id *uid=node->pkt->pkt.user_id;
 
-	    if((uid->is_expired || uid->is_revoked)
+	    if(pk && (uid->is_expired || uid->is_revoked)
 	       && !(opt.list_options&LIST_SHOW_UNUSABLE_UIDS))
 	      {
 		skip_sigs=1;
@@ -787,13 +850,13 @@ list_keyblock_print ( KBNODE keyblock, int secret, int fpr, void *opaque )
 		printf("uid%*s%s ",indent,"",validity);
 	      }
 	    else
-	      printf("uid%*s",keystrlen()+10,"");
+	      printf("uid%*s", (int)keystrlen()+10,"");
 
             print_utf8_string( stdout, uid->name, uid->len );
 	    putchar('\n');
 
 	    if((opt.list_options&LIST_SHOW_PHOTOS) && uid->attribs!=NULL)
-	      show_photos(uid->attribs,uid->numattribs,pk,sk);
+	      show_photos(uid->attribs,uid->numattribs,pk,sk,uid);
 	}
 	else if( node->pkt->pkttype == PKT_PUBLIC_SUBKEY )
 	  {
@@ -922,7 +985,7 @@ list_keyblock_print ( KBNODE keyblock, int secret, int fpr, void *opaque )
 		size_t n;
 		char *p = get_user_id( sig->keyid, &n );
                 print_utf8_string( stdout, p, n );
-		m_free(p);
+		xfree(p);
 	    }
 	    putchar('\n');
 
@@ -945,6 +1008,29 @@ list_keyblock_print ( KBNODE keyblock, int secret, int fpr, void *opaque )
     putchar('\n');
 }
 
+void
+print_revokers(PKT_public_key *pk)
+{
+  /* print the revoker record */
+  if( !pk->revkey && pk->numrevkeys )
+    BUG();
+  else
+    {
+      int i,j;
+
+      for (i=0; i < pk->numrevkeys; i++)
+	{
+	  byte *p;
+
+	  printf ("rvk:::%d::::::", pk->revkey[i].algid);
+	  p = pk->revkey[i].fpr;
+	  for (j=0; j < 20; j++, p++ )
+	    printf ("%02X", *p);
+	  printf (":%02x%s:\n", pk->revkey[i].class,
+		  (pk->revkey[i].class&0x40)?"s":"");
+	}
+    }
+}
 
 static void
 list_keyblock_colon( KBNODE keyblock, int secret, int fpr )
@@ -1009,7 +1095,7 @@ list_keyblock_colon( KBNODE keyblock, int secret, int fpr )
             putchar( get_ownertrust_info(pk) );
 	    putchar(':');
     }
-    
+
     if (opt.fixed_list_mode) {
         /* do not merge the first uid with the primary key */
         putchar(':');
@@ -1030,6 +1116,8 @@ list_keyblock_colon( KBNODE keyblock, int secret, int fpr )
           putchar(':'); /* End of field 15. */
         }
         putchar('\n');
+	if(pk)
+	  print_revokers(pk);
         if( fpr )
             print_fingerprint( pk, sk, 0 );
         if( opt.with_key_data )
@@ -1037,7 +1125,6 @@ list_keyblock_colon( KBNODE keyblock, int secret, int fpr )
         any = 1;
     }
 
-
     for( kbctx=NULL; (node=walk_kbnode( keyblock, &kbctx, 0)) ; ) {
 	if( node->pkt->pkttype == PKT_USER_ID && !opt.fast_list_mode ) {
 	    PKT_user_id *uid=node->pkt->pkt.user_id;
@@ -1229,7 +1316,7 @@ list_keyblock_colon( KBNODE keyblock, int secret, int fpr )
 
 		fflush(stdout);
 		if(opt.no_sig_cache)
-		  signer_pk=m_alloc_clear(sizeof(PKT_public_key));
+		  signer_pk=xmalloc_clear(sizeof(PKT_public_key));
 
 		rc = check_key_signature2( keyblock, node, NULL, signer_pk,
 					   NULL, NULL, NULL );
@@ -1281,21 +1368,17 @@ list_keyblock_colon( KBNODE keyblock, int secret, int fpr )
 		size_t n;
 		char *p = get_user_id( sig->keyid, &n );
                 print_string( stdout, p, n, ':' );
-		m_free(p);
+		xfree(p);
 	    }
-            printf(":%02x%c:", sig->sig_class,sig->flags.exportable?'x':'l');
+            printf(":%02x%c::", sig->sig_class,sig->flags.exportable?'x':'l');
 
 	    if(opt.no_sig_cache && opt.check_sigs && fprokay)
 	      {
-		printf(":");
-
 		for (i=0; i < fplen ; i++ )
 		  printf ("%02X", fparray[i] );
-
-		printf(":");
 	      }
 
-	    printf("\n");
+	    printf(":::%d:\n", sig->digest_algo);
 
 	    if(opt.show_subpackets)
 	      print_subpackets_colon(sig);
@@ -1315,15 +1398,16 @@ list_keyblock_colon( KBNODE keyblock, int secret, int fpr )
  * Reorder the keyblock so that the primary user ID (and not attribute
  * packet) comes first.  Fixme: Replace this by a generic sort
  * function.  */
-void
-reorder_keyblock (KBNODE keyblock)
+static void
+do_reorder_keyblock (KBNODE keyblock,int attr)
 {
     KBNODE primary = NULL, primary0 = NULL, primary2 = NULL;
     KBNODE last, node;
 
     for (node=keyblock; node; primary0=node, node = node->next) {
 	if( node->pkt->pkttype == PKT_USER_ID &&
-	    !node->pkt->pkt.user_id->attrib_data &&
+	    ((attr && node->pkt->pkt.user_id->attrib_data) ||
+	     (!attr && !node->pkt->pkt.user_id->attrib_data)) &&
             node->pkt->pkt.user_id->is_primary ) {
             primary = primary2 = node;
             for (node=node->next; node; primary2=node, node = node->next ) {
@@ -1354,6 +1438,13 @@ reorder_keyblock (KBNODE keyblock)
     primary2->next = node;
 }
 
+void
+reorder_keyblock (KBNODE keyblock)
+{
+  do_reorder_keyblock(keyblock,1);
+  do_reorder_keyblock(keyblock,0);
+}
+
 void
 list_keyblock( KBNODE keyblock, int secret, int fpr, void *opaque )
 {
@@ -1405,14 +1496,14 @@ print_fingerprint (PKT_public_key *pk, PKT_secret_key *sk, int mode )
       {
 	if(sk)
 	  {
-	    PKT_secret_key *primary_sk=m_alloc_clear(sizeof(*primary_sk));
+	    PKT_secret_key *primary_sk=xmalloc_clear(sizeof(*primary_sk));
 	    get_seckey(primary_sk,sk->main_keyid);
 	    print_fingerprint(NULL,primary_sk,mode|0x80);
 	    free_secret_key(primary_sk);
 	  }
 	else
 	  {
-	    PKT_public_key *primary_pk=m_alloc_clear(sizeof(*primary_pk));
+	    PKT_public_key *primary_pk=xmalloc_clear(sizeof(*primary_pk));
 	    get_pubkey(primary_pk,pk->main_keyid);
 	    print_fingerprint(primary_pk,NULL,mode|0x80);
 	    free_public_key(primary_pk);
@@ -1428,9 +1519,9 @@ print_fingerprint (PKT_public_key *pk, PKT_secret_key *sk, int mode )
     }
     else if (mode == 2) {
         fp = NULL; /* use tty */
-        /* Translators: this should fit into 24 bytes to that the fingerprint
-         * data is properly aligned with the user ID */
 	if(primary)
+          /* TRANSLATORS: this should fit into 24 bytes to that the
+           * fingerprint data is properly aligned with the user ID */
 	  text = _(" Primary key fingerprint:");
 	else
 	  text = _("      Subkey fingerprint:");
