@@ -5,7 +5,7 @@
  *
  * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * GnuPG is distributed in the hope that it will be useful,
@@ -14,9 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
- * USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -60,11 +58,6 @@ do_check( PKT_secret_key *sk, const char *tryagain_text, int mode,
 	if( check_cipher_algo( sk->protect.algo ) ) {
 	    log_info(_("protection algorithm %d%s is not supported\n"),
 			sk->protect.algo,sk->protect.algo==1?" (IDEA)":"" );
-	    if (sk->protect.algo==CIPHER_ALGO_IDEA)
-              {
-                write_status (STATUS_RSA_OR_IDEA);
-                idea_cipher_warn (0);
-              }
 	    return G10ERR_CIPHER_ALGO;
 	}
 	if(check_digest_algo(sk->protect.s2k.hash_algo))
@@ -98,8 +91,12 @@ do_check( PKT_secret_key *sk, const char *tryagain_text, int mode,
             u16 csumc = 0;
 
 	    i = pubkey_get_npkey(sk->pubkey_algo);
-	    assert( mpi_is_opaque( sk->skey[i] ) );
-	    p = mpi_get_opaque( sk->skey[i], &ndata );
+	    if (!mpi_is_opaque (sk->skey[i]))
+              p = NULL;
+            else
+              p = mpi_get_opaque (sk->skey[i], &ndata);
+            if (!p)
+              BUG ();
             if ( ndata > 1 )
                 csumc = p[ndata-2] << 8 | p[ndata-1];
 	    data = xmalloc_secure( ndata );
@@ -112,7 +109,7 @@ do_check( PKT_secret_key *sk, const char *tryagain_text, int mode,
                    attack */
                 sk->csum = 0;
                 csum = 1;
-                if( ndata < 20 ) 
+                if( ndata < 20 )
                     log_error("not enough bytes for SHA-1 checksum\n");
                 else {
                     MD_HANDLE h = md_open (DIGEST_ALGO_SHA1, 1);
@@ -176,9 +173,12 @@ do_check( PKT_secret_key *sk, const char *tryagain_text, int mode,
                 byte *p;
                 unsigned int ndata;
 
-                assert (mpi_is_opaque (sk->skey[i]));
-                p = mpi_get_opaque (sk->skey[i], &ndata);
-                assert (ndata >= 2);
+                if (!mpi_is_opaque (sk->skey[i]))
+                  p = NULL;
+                else
+                  p = mpi_get_opaque (sk->skey[i], &ndata);
+                if (!p || !(ndata >= 2))
+                  BUG ();
                 assert (ndata == ((p[0] << 8 | p[1]) + 7)/8 + 2);
                 buffer = xmalloc_secure (ndata);
 		cipher_sync (cipher_hd);
@@ -361,10 +361,10 @@ protect_secret_key( PKT_secret_key *sk, DEK *dek )
 		    p += narr[j];
 		    xfree(bufarr[j]);
 		}
-                
+
                 if (opt.simple_sk_checksum) {
                     log_info (_("generating the deprecated 16-bit checksum"
-                              " for secret key protection\n")); 
+                              " for secret key protection\n"));
                     csum = checksum( data, ndata-2);
                     sk->csum = csum;
                     *p++ =	csum >> 8;
@@ -412,7 +412,7 @@ protect_secret_key( PKT_secret_key *sk, DEK *dek )
                     data[1] = nbits;
 		    cipher_encrypt (cipher_hd, data+2, buffer, nbytes);
 		    xfree( buffer );
-                    
+
                     mpi_free (sk->skey[i]);
                     sk->skey[i] = mpi_set_opaque (NULL, data, nbytes+2);
 		}
