@@ -5,7 +5,7 @@
  *
  * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * GnuPG is distributed in the hope that it will be useful,
@@ -14,12 +14,11 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <config.h>
-#include <stdio.h>
+#include <config.h> 
+#include <stdio.h> 
 #include <stdlib.h>
 #include "mpi-internal.h"
 #include "longlong.h"
@@ -31,6 +30,7 @@
  *
  * FIXME: This code is VERY ugly!
  */
+#if 0 /* Code is not used */
 int
 mpi_getbyte( MPI a, unsigned idx )
 {
@@ -48,14 +48,19 @@ mpi_getbyte( MPI a, unsigned idx )
     }
     return -1;
 }
+#endif /* Code is not used */
 
 
 /****************
  * Put a value at position IDX into A. idx counts from lsb to msb
  */
+/* FIXME: There is a problem with the long constants which should have
+a LL prefix or better the macros we use at other places. */
+#if 0 /* Code is not used */
 void
 mpi_putbyte( MPI a, unsigned idx, int xc )
 {
+
     int i, j;
     unsigned n;
     mpi_ptr_t ap;
@@ -67,7 +72,7 @@ mpi_putbyte( MPI a, unsigned idx, int xc )
 	limb = ap[i];
 	for( j=0; j < BYTES_PER_MPI_LIMB; j++, n++ )
 	    if( n == idx ) {
-	      #if BYTES_PER_MPI_LIMB == 4
+#if BYTES_PER_MPI_LIMB == 4
 		if( j == 0 )
 		    limb = (limb & 0xffffff00) | c;
 		else if( j == 1 )
@@ -76,7 +81,7 @@ mpi_putbyte( MPI a, unsigned idx, int xc )
 		    limb = (limb & 0xff00ffff) | (c<<16);
 		else
 		    limb = (limb & 0x00ffffff) | (c<<24);
-	      #elif BYTES_PER_MPI_LIMB == 8
+#elif BYTES_PER_MPI_LIMB == 8
 		if( j == 0 )
 		    limb = (limb & 0xffffffffffffff00) | c;
 		else if( j == 1 )
@@ -93,9 +98,9 @@ mpi_putbyte( MPI a, unsigned idx, int xc )
 		    limb = (limb & 0xff00ffffffffffff) | (c<<48);
 		else
 		    limb = (limb & 0x00ffffffffffffff) | (c<<56);
-	      #else
-		 #error please enhance this function, its ugly - i know.
-	      #endif
+#else
+#error please enhance this function, its ugly - i know.
+#endif
 		if( a->nlimbs <= i )
 		    a->nlimbs = i+1;
 		ap[i] = limb;
@@ -104,12 +109,13 @@ mpi_putbyte( MPI a, unsigned idx, int xc )
     }
     abort(); /* index out of range */
 }
+#endif /* Code is not used */
 
 
 /****************
  * Count the number of zerobits at the low end of A
  */
-unsigned
+unsigned int
 mpi_trailing_zeros( MPI a )
 {
     unsigned n, count = 0;
